@@ -17,6 +17,7 @@
 package com.google.inject.multibindings;
 
 import static com.google.common.base.Predicates.equalTo;
+import static com.google.common.primitives.Ints.MAX_POWER_OF_TWO;
 import static com.google.common.collect.Iterables.filter;
 import static com.google.common.collect.Iterables.getOnlyElement;
 import static com.google.inject.multibindings.Element.Type.MULTIBINDER;
@@ -26,6 +27,7 @@ import com.google.common.base.Objects;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import com.google.inject.AbstractModule;
 import com.google.inject.Binder;
 import com.google.inject.Binding;
@@ -34,21 +36,23 @@ import com.google.inject.Inject;
 import com.google.inject.Injector;
 import com.google.inject.Key;
 import com.google.inject.Module;
+import com.google.inject.Provider;
 import com.google.inject.TypeLiteral;
 import com.google.inject.binder.LinkedBindingBuilder;
-import com.google.inject.internal.Annotations;
 import com.google.inject.internal.Errors;
 import com.google.inject.spi.BindingTargetVisitor;
 import com.google.inject.spi.Dependency;
 import com.google.inject.spi.HasDependencies;
 import com.google.inject.spi.Message;
 import com.google.inject.spi.ProviderInstanceBinding;
+import com.google.inject.spi.ProviderWithDependencies;
 import com.google.inject.spi.ProviderWithExtensionVisitor;
 import com.google.inject.spi.Toolable;
 import com.google.inject.util.Types;
 
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Type;
+import java.util.Collection;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -76,6 +80,8 @@ import java.util.Set;
  *   public SnackMachine(Set&lt;Snack&gt; snacks) { ... }
  * }</code></pre>
  *
+ * If desired, {@link Collection}{@code <Provider<Snack>>} can also be injected.
+ *
  * <p>Contributing multibindings from different modules is supported. For
  * example, it is okay for both {@code CandyModule} and {@code ChipsModule}
  * to create their own {@code Multibinder<Snack>}, and to each contribute
@@ -115,11 +121,7 @@ public abstract class Multibinder<T> {
    * itself bound with no binding annotation.
    */
   public static <T> Multibinder<T> newSetBinder(Binder binder, TypeLiteral<T> type) {
-    binder = binder.skipSources(RealMultibinder.class, Multibinder.class);
-    RealMultibinder<T> result = new RealMultibinder<T>(binder, type,
-        Key.get(Multibinder.<T>setOf(type)));
-    binder.install(result);
-    return result;
+    return newRealSetBinder(binder, Key.get(type));
   }
 
   /**
@@ -127,7 +129,7 @@ public abstract class Multibinder<T> {
    * itself bound with no binding annotation.
    */
   public static <T> Multibinder<T> newSetBinder(Binder binder, Class<T> type) {
-    return newSetBinder(binder, TypeLiteral.get(type));
+    return newRealSetBinder(binder, Key.get(type));
   }
 
   /**
@@ -136,11 +138,7 @@ public abstract class Multibinder<T> {
    */
   public static <T> Multibinder<T> newSetBinder(
       Binder binder, TypeLiteral<T> type, Annotation annotation) {
-    binder = binder.skipSources(RealMultibinder.class, Multibinder.class);
-    RealMultibinder<T> result = new RealMultibinder<T>(binder, type,
-        Key.get(Multibinder.<T>setOf(type), annotation));
-    binder.install(result);
-    return result;
+    return newRealSetBinder(binder, Key.get(type, annotation));
   }
 
   /**
@@ -149,7 +147,7 @@ public abstract class Multibinder<T> {
    */
   public static <T> Multibinder<T> newSetBinder(
       Binder binder, Class<T> type, Annotation annotation) {
-    return newSetBinder(binder, TypeLiteral.get(type), annotation);
+    return newRealSetBinder(binder, Key.get(type, annotation));
   }
 
   /**
@@ -158,9 +156,25 @@ public abstract class Multibinder<T> {
    */
   public static <T> Multibinder<T> newSetBinder(Binder binder, TypeLiteral<T> type,
       Class<? extends Annotation> annotationType) {
+    return newRealSetBinder(binder, Key.get(type, annotationType));
+  }
+
+  /**
+   * Returns a new multibinder that collects instances of the key's type in a {@link Set} that is
+   * itself bound with the annotation (if any) of the key.
+   * @since 4.0
+   */
+  public static <T> Multibinder<T> newSetBinder(Binder binder, Key<T> key) {
+    return newRealSetBinder(binder, key);
+  }
+
+  /**
+   * Implementation of newSetBinder.
+   */
+  static <T> RealMultibinder<T> newRealSetBinder(Binder binder, Key<T> key) {
     binder = binder.skipSources(RealMultibinder.class, Multibinder.class);
-    RealMultibinder<T> result = new RealMultibinder<T>(binder, type,
-        Key.get(Multibinder.<T>setOf(type), annotationType));
+    RealMultibinder<T> result = new RealMultibinder<T>(binder, key.getTypeLiteral(),
+        key.ofType(setOf(key.getTypeLiteral())));
     binder.install(result);
     return result;
   }
@@ -171,7 +185,7 @@ public abstract class Multibinder<T> {
    */
   public static <T> Multibinder<T> newSetBinder(Binder binder, Class<T> type,
       Class<? extends Annotation> annotationType) {
-    return newSetBinder(binder, TypeLiteral.get(type), annotationType);
+    return newSetBinder(binder, Key.get(type, annotationType));
   }
 
   @SuppressWarnings("unchecked") // wrapping a T in a Set safely returns a Set<T>
@@ -180,6 +194,23 @@ public abstract class Multibinder<T> {
     return (TypeLiteral<Set<T>>) TypeLiteral.get(type);
   }
 
+  @SuppressWarnings("unchecked")
+  static <T> TypeLiteral<Collection<Provider<T>>> collectionOfProvidersOf(
+      TypeLiteral<T> elementType) {
+    Type providerType = Types.providerOf(elementType.getType());
+    Type type = Types.newParameterizedType(Collection.class, providerType);
+    return (TypeLiteral<Collection<Provider<T>>>) TypeLiteral.get(type);
+  }
+
+  @SuppressWarnings("unchecked")
+  static <T> TypeLiteral<Collection<javax.inject.Provider<T>>> collectionOfJavaxProvidersOf(
+      TypeLiteral<T> elementType) {
+    Type providerType =
+        Types.newParameterizedType(javax.inject.Provider.class, elementType.getType());
+    Type type = Types.newParameterizedType(Collection.class, providerType);
+    return (TypeLiteral<Collection<javax.inject.Provider<T>>>) TypeLiteral.get(type);
+  }
+
   /**
    * Configures the bound set to silently discard duplicate elements. When multiple equal values are
    * bound, the one that gets included is arbitrary. When multiple modules contribute elements to
@@ -230,6 +261,8 @@ public abstract class Multibinder<T> {
     private final TypeLiteral<T> elementType;
     private final String setName;
     private final Key<Set<T>> setKey;
+    private final Key<Collection<Provider<T>>> collectionOfProvidersKey;
+    private final Key<Collection<javax.inject.Provider<T>>> collectionOfJavaxProvidersKey;
     private final Key<Boolean> permitDuplicatesKey;
 
     /* the target injector's binder. non-null until initialization, null afterwards */
@@ -246,6 +279,8 @@ public abstract class Multibinder<T> {
       this.binder = checkNotNull(binder, "binder");
       this.elementType = checkNotNull(elementType, "elementType");
       this.setKey = checkNotNull(setKey, "setKey");
+      this.collectionOfProvidersKey = setKey.ofType(collectionOfProvidersOf(elementType));
+      this.collectionOfJavaxProvidersKey = setKey.ofType(collectionOfJavaxProvidersOf(elementType));
       this.setName = RealElement.nameOf(setKey);
       this.permitDuplicatesKey = Key.get(Boolean.class, named(toString() + " permits duplicates"));
     }
@@ -254,18 +289,29 @@ public abstract class Multibinder<T> {
       checkConfiguration(!isInitialized(), "Multibinder was already initialized");
 
       binder.bind(setKey).toProvider(this);
+      binder.bind(collectionOfProvidersKey).toProvider(
+          new RealMultibinderCollectionOfProvidersProvider());
+
+      // The collection this exposes is internally an ImmutableList, so it's OK to massage
+      // the guice Provider to javax Provider in the value (since the guice Provider implements
+      // javax Provider).
+      @SuppressWarnings("unchecked")
+      Key key = (Key) collectionOfProvidersKey;
+      binder.bind(collectionOfJavaxProvidersKey).to(key);
     }
 
-    @Override
-    public Multibinder<T> permitDuplicates() {
+    @Override public Multibinder<T> permitDuplicates() {
       binder.install(new PermitDuplicatesModule(permitDuplicatesKey));
       return this;
     }
+    
+    Key<T> getKeyForNewItem() {
+      checkConfiguration(!isInitialized(), "Multibinder was already initialized");
+      return Key.get(elementType, new RealElement(setName, MULTIBINDER, ""));
+    }
 
     @Override public LinkedBindingBuilder<T> addBinding() {
-      checkConfiguration(!isInitialized(), "Multibinder was already initialized");
-
-      return RealElement.addBinding(binder, MULTIBINDER, elementType, setName);
+      return binder.bind(getKeyForNewItem());
     }
 
     /**
@@ -275,13 +321,17 @@ public abstract class Multibinder<T> {
      */
     @Toolable @Inject void initialize(Injector injector) {
       List<Binding<T>> bindings = Lists.newArrayList();
+      Set<Indexer.IndexedBinding> index = Sets.newHashSet();
+      Indexer indexer = new Indexer(injector);
       List<Dependency<?>> dependencies = Lists.newArrayList();
       for (Binding<?> entry : injector.findBindingsByType(elementType)) {
         if (keyMatches(entry.getKey())) {
           @SuppressWarnings("unchecked") // protected by findBindingsByType()
           Binding<T> binding = (Binding<T>) entry;
-          bindings.add(binding);
-          dependencies.add(Dependency.get(binding.getKey()));
+          if (index.add(binding.acceptTargetVisitor(indexer))) {
+            bindings.add(binding);
+            dependencies.add(Dependency.get(binding.getKey()));
+          }
         }
       }
 
@@ -291,6 +341,16 @@ public abstract class Multibinder<T> {
       this.binder = null;
     }
 
+    // This is forked from com.google.common.collect.Maps.capacity 
+    private static int mapCapacity(int numBindings) {
+      if (numBindings < 3) {
+        return numBindings + 1;
+      } else  if (numBindings < MAX_POWER_OF_TWO) {
+        return (int) (numBindings / 0.75F + 1.0F);
+      }
+      return Integer.MAX_VALUE;
+    }
+
     boolean permitsDuplicates(Injector injector) {
       return injector.getBindings().containsKey(permitDuplicatesKey);
     }
@@ -309,10 +369,12 @@ public abstract class Multibinder<T> {
     public Set<T> get() {
       checkConfiguration(isInitialized(), "Multibinder is not initialized");
 
-      Map<T, Binding<T>> result = new LinkedHashMap<T, Binding<T>>();
+      Map<T, Binding<T>> result = new LinkedHashMap<T, Binding<T>>(mapCapacity(bindings.size()));
       for (Binding<T> binding : bindings) {
         final T newValue = binding.getProvider().get();
-        checkConfiguration(newValue != null, "Set injection failed due to null element");
+        checkConfiguration(newValue != null,
+            "Set injection failed due to null element bound at: %s",
+            binding.getSource());
         Binding<T> duplicateBinding = result.put(newValue, binding);
         if (!permitDuplicates && duplicateBinding != null) {
           throw newDuplicateValuesException(result, binding, newValue, duplicateBinding);
@@ -367,7 +429,9 @@ public abstract class Multibinder<T> {
         Binding<?> binding = (Binding<?>) element;
         return keyMatches(binding.getKey())
             || binding.getKey().equals(permitDuplicatesKey)
-            || binding.getKey().equals(setKey);
+            || binding.getKey().equals(setKey)
+            || binding.getKey().equals(collectionOfProvidersKey)
+            || binding.getKey().equals(collectionOfJavaxProvidersKey);
       } else {
         return false;
       }
@@ -391,13 +455,48 @@ public abstract class Multibinder<T> {
     }
 
     @Override public String toString() {
-      return new StringBuilder()
-          .append(setName)
-          .append(setName.length() > 0 ? " " : "")
-          .append("Multibinder<")
-          .append(elementType)
-          .append(">")
-          .toString();
+      return (setName.isEmpty() ? "" : setName + " ") + "Multibinder<" + elementType + ">";
+    }
+
+    final class RealMultibinderCollectionOfProvidersProvider
+        implements ProviderWithDependencies<Collection<Provider<T>>> {
+      @Override public Collection<Provider<T>> get() {
+        checkConfiguration(isInitialized(), "Multibinder is not initialized");
+        int size = bindings.size();
+        @SuppressWarnings("unchecked")  // safe because we only put Provider<T> into it.
+        Provider<T>[] providers = new Provider[size];
+        for (int i = 0; i < size; i++) {
+          providers[i] = bindings.get(i).getProvider();
+        }
+        return ImmutableList.copyOf(providers);
+      }
+
+      @Override public Set<Dependency<?>> getDependencies() {
+        if (!isInitialized()) {
+          return ImmutableSet.<Dependency<?>>of(Dependency.get(Key.get(Injector.class)));
+        }
+        ImmutableSet.Builder<Dependency<?>> setBuilder = ImmutableSet.builder();
+        for (Dependency<?> dependency : dependencies) {
+          Key key = dependency.getKey();
+          setBuilder.add(
+              Dependency.get(key.ofType(Types.providerOf(key.getTypeLiteral().getType()))));
+        }
+        return setBuilder.build();
+      }
+
+      Key getCollectionKey() {
+        return RealMultibinder.this.collectionOfProvidersKey;
+      }
+
+      @Override public boolean equals(Object o) {
+        return o instanceof Multibinder.RealMultibinder.RealMultibinderCollectionOfProvidersProvider
+            && ((Multibinder.RealMultibinder.RealMultibinderCollectionOfProvidersProvider) o)
+                .getCollectionKey().equals(getCollectionKey());
+      }
+
+      @Override public int hashCode() {
+        return getCollectionKey().hashCode();
+      }
     }
   }
 
@@ -413,8 +512,7 @@ public abstract class Multibinder<T> {
       this.key = key;
     }
 
-    @Override
-    protected void configure() {
+    @Override protected void configure() {
       bind(key).toInstance(true);
     }
 
